/*
 *  This file is part of the KDE System Control Tool,
 *  Copyright (C)1999 Thorsten Westheider <twesthei@physik.uni-bielefeld.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/

#ifndef __IDEDRIVE_H
#define __IDEDRIVE_H

#include <qstring.h>

#include "device.h"
#include "driveinfo.h"


class IDEDrive : public Device
{
  public:
  
    enum idedrvclass { IDEClassDisk,
                       IDEClassFloppy,
		       IDEClassCDROM,
		       IDEClassUnknown };
  
    IDEDrive(const QString& modelname, idedrvclass, DriveInfo *);
	       
    ~IDEDrive() {}
    
    virtual bustype        busType()        const { return BusIDE;      }
 
    virtual QString        deviceClassName();
    virtual QString        deviceName()           { return _modelname;  }
    virtual devstate       deviceState();
    virtual QString        vendorName()           { return _vendorname; }

	    DriveInfo      *driveInfo()           { return _driveinfo;  }
	    PartitionInfo  *firstPartition();
	    PartitionInfo  *nextPartition();

  private:
  
    QString      _vendorname, _modelname;
    QString      _devicestr;
    idedrvclass  _class;
    DriveInfo    *_driveinfo;
    
    void  lookupData();
};


#endif

