/*
 *  This file is part of the KDE System Control Tool,
 *  Copyright (C)1999 Thorsten Westheider <twesthei@physik.uni-bielefeld.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************************/

#include <stdio.h>

#include <qfile.h>

#include <kdebug.h>

#include "driveinfo.h"
#include "idedrvscanner.h"


IDEDriveScanner::IDEDriveScanner()
{
  
}


IDEDriveScanner::~IDEDriveScanner()
{
  
}


/*
 * Public methods
 *****************/
 
Device  *IDEDriveScanner::firstDevice()
{
  _drvid = 'a';
    
  return nextDevice();
}


Device  *IDEDriveScanner::nextDevice()
{
  IDEDrive  *drive;
  
  do
  {
    drive = device();
    _drvid++;
  } while (!drive && (_drvid <= 'd')); 
    
  return drive;
}


/*
 * Private methods
 ******************/

IDEDrive  *IDEDriveScanner::device()
{
  QString                path = QString("/proc/ide/hd")+_drvid;
  QString                modelname, media;
  FILE                   *idefhd;
  char                   buffer[256];
  IDEDrive::idedrvclass  drvclass;
  DriveInfo::drivetype   drvtype;
  
  if ((idefhd = fopen(QFile::encodeName(path+"/model"), "r")))
  {
    fgets(buffer, sizeof(buffer), idefhd);
    fclose(idefhd);
    
    modelname = QString(buffer).simplifyWhiteSpace();
    
    if ((idefhd = fopen(QFile::encodeName(path+"/media"), "r")))
    {
      fgets(buffer, sizeof(buffer), idefhd);
      fclose(idefhd);
      
      media = QString(buffer).simplifyWhiteSpace();

      if      (media == "disk")   { drvclass = IDEDrive::IDEClassDisk;    drvtype = DriveInfo::DriveHD;    }
      else if (media == "floppy") { drvclass = IDEDrive::IDEClassFloppy;  drvtype = DriveInfo::DriveHD;    }
      else if (media == "cdrom")  { drvclass = IDEDrive::IDEClassCDROM;   drvtype = DriveInfo::DriveCDROM; }
      else                        { drvclass = IDEDrive::IDEClassUnknown; drvtype = DriveInfo::DriveNone;  }
      
      return new IDEDrive(modelname, drvclass, new DriveInfo(QString("hd")+_drvid, drvtype));

    } else kdError() << perror << path << "/media" << endl;
  }
  
  return 0L;
}





