/***************************************************************************
                          TopWidget.h  -  description
                             -------------------
    begin                : Sun Oct 3 1999
    copyright            : (C) 1997-99 by Peter Putzer
    email                : putzer@kde.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2.                              *
 *                                                                         *
 ***************************************************************************/

#ifndef KSV_TOPWIDGET_H
#define KSV_TOPWIDGET_H

#include <kmainwindow.h>

class QString;
class QCheckBox;

class KAction;
class KToggleAction;
class KMenuBar;
class KStatusBar;
class KToolBar;

class ActionList;
class KSVAction;
class ServiceDlg;
class KSVConfig;
class KSVItem;
class KSVContent;
class KSVPreferences;
class RunlevelAuthIcon;

class KSVTopLevel : public KMainWindow
{
  Q_OBJECT

public:
  KSVTopLevel();
  virtual ~KSVTopLevel();

public slots:
  virtual void show();
  void setCaption (bool changed);

private:
  virtual bool queryExit();
  virtual void closeEvent( QCloseEvent* e );
  void initTools();
  void initActions ();
  void initStatusBar();

private slots:
  void slotClearChanges();
  void slotAcceptChanges();
  void pushUndoAction (KSVAction*);
  void slotShowConfig();
  void slotChanged();
  void slotReadConfig();
  void pasteAppend ();
  void toggleLog();
  void toggleTools();
  void toggleStatus();
  void saveOptions();
  void slotUpdateRunning (const QString&);
  void editCut();
  void editCopy();
  void editPaste();
  void editUndo();
  void editRedo();
  void saveAs ();
  void load ();
  void print();
  void printLog();
  void setChanged( bool val = TRUE );
  void setCopy( bool val = FALSE );
  void setCut( bool val = FALSE );
  void setPaste( bool val = FALSE );
  void setUndo( bool val = FALSE );
  void setRedo (bool val = false);
  void properties();
  void scriptProperties();
  void setLog( bool val );
  void setStatus( bool val );
  void setTools( bool val );
  void catchCannotGenerateNumber();

  void configureKeys ();
  void configureToolbars ();

  void enableUndo();
  void disableUndo();
  void enableRedo();
  void disableRedo();

  void dispatchEditService (bool);
  void dispatchStartService (bool);
  void dispatchStopService (bool);
  void dispatchRestartService (bool);

  void dispatchEdit ();

  /**
   * initialize the view (plus some other things)
   */
  void initView();

  void slotSaveLog();
  void enableLogActions();

  void checkScriptMenu (QPopupMenu*, int);

  inline void setUnsavedData (bool val) { mChanged = val; }

  void toggleRunlevel (int index);

  void updateColors ();
  void updateServicesPath ();
  void updateRunlevelsPath ();

  void writingEnabled (bool);

private:
  KSVConfig* mConfig;
  KSVContent* mView;

  KSVPreferences* mPreferences;

  KAction *mEditUndo, *mEditRedo, *mEditCut, *mEditCopy, *mEditPaste, *mPasteAppend;
  KAction *mFileRevert, *mFileLoad, *mFilePrint, *mFilePrintLog,
    *mFileSave, *mFileSaveAs, *mFileSaveLog, *mFileQuit;
  
  KToggleAction *mToolsStartService, *mToolsStopService, *mToolsRestartService, *mToolsEditService;
  KToggleAction *mOptionsTools, *mOptionsStatus, *mOptionsToggleLog;

  bool mChanged;

  ActionList* mUndoList;
  ActionList* mRedoList;

  ServiceDlg* mStartDlg;
  ServiceDlg* mStopDlg;
  ServiceDlg* mRestartDlg;
  ServiceDlg* mEditDlg;

  RunlevelAuthIcon* mAuth;
  QCheckBox** mVisible;
};

#endif
